dojo.provide('ibm.DynamicTreeModel');

dojo.require("dojox.data.QueryReadStore");
dojo.require("dijit.Tree");

dojo.declare('ibm.DynamicTreeModel',dijit.tree.ForestStoreModel,{

	// Methods for traversing hierarchy

	mayHaveChildren: function(/*dojo.data.Item*/ item){
	
		return this.store.mayHaveChildren(item);
	},

	getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError){
		// summary
		// 		Calls onComplete() with array of child items of given parent item, all loaded.
		if(parentItem === this.root) {
			if(this.root.children) {
				// already loaded, just return
				callback(this.root.children);
			} else {
				this.store.fetch({
					query: this.query,
					onComplete: dojo.hitch(this, function(items){
						this.root.children = items;
						if (ROOT_TREE_STORE_ITEMS == null)
							ROOT_TREE_STORE_ITEMS = items;
						callback(items);
					}),
					onError: onError
				});
			}
		} else {
			var typeVal = this.store.getValue(parentItem, 'type');
			if (typeVal == 'publicsite') {
				this.store.fetch({
					title: this.store.getValue(parentItem, 'title'),
					onComplete: dojo.hitch(this, function(items){
						this.root.children = items;
						callback(items);
					}),
					onError: onError});
			} else {
				this.store.fetch({
					link: encodeURI(unescape(this.store.getValue(parentItem, 'link'))),
					onComplete: dojo.hitch(this, function(items){
						this.root.children = items;
						callback(items);
					}),
					onError: onError});
			}
			
//			this.inherited(arguments);
		}
	}
});


dojo.provide('ibm.DynamicReadStore');
dojo.declare('ibm.DynamicReadStore', dojox.data.QueryReadStore, {
    fetch: function(request) {
		
		if (request.link)
			request.serverQuery = {
					link: request.link
					};
		else if (request.title)
			request.serverQuery = {
					title: request.title
					};
        return this.inherited('fetch', arguments);
	},
    
	mayHaveChildren: function(item) {
		if(item.root)
			return true;

		var typeVal = this.getValue(item, 'type');
		if(typeVal == 'site' || typeVal == 'localsite' || typeVal == 'watchedsite' || typeVal == 'publicsite')
			return true;

		return false;
	}
});

dojo.provide('ibm._CheckTreeNode');
dojo.declare('ibm._CheckTreeNode', dijit._TreeNode, {
	isExpandable: false,
	checkbox: null,
	postCreate: function(){
		// set label, escaping special characters
		this.setLabelNode(this.label);

		// set expand icon for leaf
		this._setExpando();

		// set icon and label class based on item
		this._updateItemClasses(this.item);

		if(this.isExpandable){
			dijit.setWaiState(this.labelNode, "expanded", this.isExpanded);
		}
	},

	setLabelNode: function(label) {
		
		var typeVal = (this.item!=null && !this.item.root) ? this.tree.model.store.getValue(this.item, 'type') : "";
        if(this.item.root || typeVal != 'category')
            return;
		
        if (this.tree.model.store.getValue(this.item, 'installed') == 'true')
        	return;
        
		var chkBox = new dijit.form.CheckBox();
		chkBox.id = 'chk_' + this.tree.model.store.getValue(this.item, 'uid');
		chkBox.link = this.tree.model.store.getValue(this.item, 'link');
		chkBox.parentNode = this;
		var chk = chkBox.domNode;
		
		chk.type = 'checkbox';
		
		//**************************************************************
		//* Method for handling a checkbox click, to update the appropriate 
		//* list of checkboxes selected
		//**************************************************************
		chkBox.label = this.tree.model.store.getValue(this.item, 'title');
		chkBox.dependency = this.tree.model.store.getValue(this.item, 'dependency');
		
		chkBox.onClick = function(event) {
			
			onClick_Category_TreeNode(this);
			
			IS_INSTALL_DISABLED = SelectedCategoriesEntries.count <= 0;
			
			window.updateButtons();
		};
		
		dojo.place(chk, this.iconNode, 'replace');

		this.checkNode = chk;
		this.checkbox = chkBox;
	},
	
	destroy: function(/*Boolean*/ preserveDom){
		
		if (this.checkbox)
			this.checkbox.destroy();
		
		this.inherited(arguments);
	},

	getCheckBox: function() {
		return this.checkbox;
	}
});

dojo.provide('ibm.widget.CheckTree');
dojo.declare('ibm.widget.CheckTree', dijit.Tree, {
	_clickTarget: null,
	_createTreeNode: function(args) {
		return new ibm._CheckTreeNode(args);
	},
	
	getLabel: function(/*dojo.data.Item*/ item){
		return unescape(this.model.getLabel(item));	// String
	},

	getIconClass: function(item, opened) {
		var typeVal = (item!=null && !item.root) ? this.model.store.getValue(item, 'type') : "";
		if (item == null || item.root)
			return this.inherited(arguments);
		else if (typeVal == 'publicsite' || typeVal == 'site' ||
				typeVal == 'watchedsite')
			return (this.model.store.getValue(item, "children") != null) ?
					(opened ? "Tree_Icons siteFolderOpenedIcon" : "Tree_Icons siteFolderClosedIcon") :
						"noteIcon";
		else if (typeVal == 'localsite')
			return (this.model.store.getValue(item, "children") != null) ?
					(opened ? "Tree_Icons localsiteFolderOpenedIcon" : "Tree_Icons localsiteFolderClosedIcon") :
						"noteIcon";
		else if (typeVal == 'category' 
			&& this.model.store.getValue(item, 'installed') == 'true' && this.model.store.getValue(item, 'update') == 'false')
			return "Tree_Icons siteCategoryIcon";
		else if (typeVal == 'category' 
			&& this.model.store.getValue(item, 'installed') == 'true' && this.model.store.getValue(item, 'update') == 'true')
			return "Tree_Icons siteCategoryIconUpdate";
		return '';
	},

	getLabelClass: function(item, opened) {
		var typeVal = (item!=null && !item.root) ? this.model.store.getValue(item, 'type') : "";
		if (item==null || item.root || typeVal == 'site' || typeVal == 'localsite' || typeVal == 'publicsite' || typeVal == 'watchedsite')
			return this.inherited(arguments);

		return '';
	},
	
	_onNodeFocus: function(/*dijit._Widget*/ node){
		// summary:
		//		Called when a TreeNode gets focus, either by user clicking
		//		it, or programatically by arrow key handling code.
		// description:
		//		It marks that the current node is the selected one, and the previously
		//		selected node no longer is.
		this.inherited(arguments);
		
		this.onNodeFocusEvent(node);
	},
	
	onNodeFocusEvent: function(/*dijit._Widget*/ node){
		// @snehpaul - User can extended this method for UI updates...
	},

	_onItemChange: function(/*Item*/ item){
		// summary:
		//		Processes notification of a change to an item's scalar values like label
		var model = this.model,
			identity = model.getIdentity(item),
			nodes = this._itemNodesMap[identity];

		if(nodes){
			var self = this;
			dojo.forEach(nodes,function(node){
				// Custom call to add checkbox
				node.setLabelNode(this.getLabel(item));
				
				node.attr({
					label: self.getLabel(item),
					tooltip: self.getTooltip(item)
				});
				node._updateItemClasses(item);
			});
		}
	}
});
